import os
import tkinter as tk
from tkinter import filedialog
import pypimaker.ui.navigation as navigation
from pypimaker.ui.AuthorInfoSelector import AuthorInfoSelector
from pypimaker.ui.DialogWindow import ActionDialogWindow, InfoDialogWindow

MISMATCHED_SELECTION_TEXT = "Mismatched folder name and project name.\nAre you sure you selected correctly?"
NO_PYTHON_FILES_IN_PROJECT_FOLDER_TEXT = "No Python files found in folder.\nSelect a different folder."
PROJECT_DESCRIPTION_TEXT = 'One-sentence Project Description (e.g., "Software designed for..."):'
PROJECT_FOLDER_NOT_CHOSEN_TEXT = "You must select your project \nusing the Browse button."
PROJECT_NAME_NOT_CHOSEN_TEXT = "Project Name is a required field.\nUse folder name as project name?"
SELECT_FOLDER_TEXT = "Select the top-level folder containing your Python project"
PROJECT_NAME_DESCRIPTION_TEXT = '(Your package will be installed with the "pip install \
project_name" command using the project name given above.)'

class OptionSelector:
	def __init__(self):
		self.window = tk.Tk()
		self.window.title("PyPI Maker")
		centerX = (self.window.winfo_screenwidth() - 650) // 2
		centerY = (self.window.winfo_screenheight() - 250) // 2
		self.window.geometry(f"650x250+{centerX}+{centerY}")

		self.filepath = ""
		self.projectName = ""
		self.authorsArray = []
		self.emailsArray = []
		self.correspondingEmail = ""
		self.githubUsername = ""
		self.shortDescription = ""
		self.intendedAudience = ""
		self.intendedAudiences = [
			"N/A",
			"Customer Service",
			"Developers",
			"Education",
			"End Users/Desktop",
			"Financial and Insurance Industry",
			"Healthcare Industry",
			"Information Technology",
			"Legal Industry",
			"Manufacturing",
			"Other Audience",
			"Religion",
			"Science/Research",
			"System Administrators",
			"Telecommunications Industry"
		]
		self.classifiers = False
		
		filepathFrame = tk.Frame(self.window)
		self.labelSelectedPath = tk.Label(self.window, text=SELECT_FOLDER_TEXT, fg="red", 
			bg="white")
		buttonSelectFolder = tk.Button(self.window, text="Browse...", 
			command=self.browseFolders)
		self.showFilepath()
		self.labelSelectedPath.pack(in_=filepathFrame, side=tk.LEFT)
		buttonSelectFolder.pack(in_=filepathFrame, side=tk.LEFT)
		
		navigation = tk.Frame(self.window)
		labelIntendedAudienceSelect = tk.Label(self.window, text="Intended Audience:")
		self.menuIntendedAudienceSelectVar = tk.StringVar(self.window)
		self.menuIntendedAudienceSelectVar.set("Select...")
		menuIntendedAudienceSelect = tk.OptionMenu(self.window, self.menuIntendedAudienceSelectVar,
			*self.intendedAudiences)
		labelIntendedAudienceSelect.pack(in_=navigation, side=tk.LEFT)
		menuIntendedAudienceSelect.pack(in_=navigation, side=tk.LEFT)
		
		frameProjectName = tk.Frame(self.window)
		labelProjectName = tk.Label(self.window, text="Project Name:")
		self.entryProjectName = tk.Entry(self.window, width=25)
		labelProjectName.pack(in_=frameProjectName, side=tk.LEFT)
		self.entryProjectName.pack(in_=frameProjectName, side=tk.LEFT)
		
		labelProjectNameDescription = tk.Label(self.window, text=PROJECT_NAME_DESCRIPTION_TEXT, font=("Helvetica 9", 10))
		
		frameGithubUsername = tk.Frame(self.window)
		labelGithubUsername = tk.Label(self.window, text="Project GitHub Username:")
		self.entryGithubUsername = tk.Entry(self.window, width=25)
		labelGithubUsername.pack(in_=frameGithubUsername, side=tk.LEFT)
		self.entryGithubUsername.pack(in_=frameGithubUsername, side=tk.LEFT)
		
		labelProjectDescription = tk.Label(self.window, text=PROJECT_DESCRIPTION_TEXT)
		self.entryProjectDescription = tk.Entry(self.window, width=50)
		
		self.includeAuthorNames = tk.BooleanVar(value=True)
		checkboxIncludeAuthorNames = tk.Checkbutton(self.window, text="Include author name(s)?", 
			variable=self.includeAuthorNames, onvalue=True, offvalue=False)
				
		buttonDone = tk.Button(self.window, text="Done", command=self.checkDone, fg="blue")
				
		filepathFrame.pack()
		navigation.pack()
		frameProjectName.pack()
		labelProjectNameDescription.pack()
		frameGithubUsername.pack()
		labelProjectDescription.pack()
		self.entryProjectDescription.pack()
		checkboxIncludeAuthorNames.pack()
		buttonDone.pack()
		
		self.window.protocol("WM_DELETE_WINDOW", quit)
		self.window.bind("<Return>", self.checkDoneWithReturnKey)
		
		self.window.mainloop()

	def browseFolders(self):
		chosenFolder = filedialog.askdirectory(initialdir=self.filepath, title=SELECT_FOLDER_TEXT)
		if chosenFolder not in ["", " ", "/"]:
			if not navigation.hasPyFiles(chosenFolder):
				InfoDialogWindow("No Python files found", NO_PYTHON_FILES_IN_PROJECT_FOLDER_TEXT)
				return
			self.filepath = chosenFolder
		else:
			self.filepath = ""
		self.showFilepath()
		self.window.focus_force()
	
	def checkDone(self):
		if self.filepath in ["", " ", "/"]:
			InfoDialogWindow("Missing info", PROJECT_FOLDER_NOT_CHOSEN_TEXT)
			return
		
		self.projectName = self.entryProjectName.get()
		if self.projectName == "":
			ActionDialogWindow("Missing info", PROJECT_NAME_NOT_CHOSEN_TEXT, 
				positiveAction=self.finishWithFolderNameAsProjectName)
			return
		
		chosenFolder = self.filepath.split("/")[-1]
		if chosenFolder != self.projectName:
			ActionDialogWindow("Check your selection", MISMATCHED_SELECTION_TEXT, 
				positiveAction=self.done)
			return
		
		self.done()
	
	def checkDoneWithReturnKey(self, event):        
		self.checkDone()
	
	def closeWindow(self):
		self.window.destroy()
		self.window.quit()
	
	def done(self):
		self.githubUsername = self.entryGithubUsername.get()
		self.shortDescription = self.entryProjectDescription.get()
		self.classifier = self.menuIntendedAudienceSelectVar.get()
		self.classifiers = self.classifier not in ["N/A", "Select..."]
		
		if self.includeAuthorNames.get():
			self.getNamesAndEmails()
		else:
			self.closeWindow()
	
	def finishWithFolderNameAsProjectName(self):
		chosenFolder = self.filepath.split("/")[-1]
		self.projectName = chosenFolder
		self.done()
	
	def getNamesAndEmails(self):
		self.closeWindow()
		authorInfoSelector = AuthorInfoSelector()
		self.authorsArray = authorInfoSelector.names
		self.emailsArray = authorInfoSelector.emails
		self.correspondingEmail = authorInfoSelector.correspondingEmail
	
	def showFilepath(self):
		if self.filepath in ["", " ", "/"]:
			self.labelSelectedPath.configure(text=SELECT_FOLDER_TEXT, fg="red", bg="white")
		else:
			displayedFilepath = self.filepath
			if len(displayedFilepath) > 50:
				displayedFilepath = "..." + displayedFilepath[-50:]
			self.labelSelectedPath.configure(text=displayedFilepath, bg="white", fg="black")



import unittest
import pypimaker.files as files

# This unit test uses Python's built-in unit testing framework
# See https://docs.python.org/3/library/unittest.html for more information

class FilesTest(unittest.TestCase):
	def test_getNamesAndEmailsVerticallySpaced_spacesInfoProperly(self):
		authorsArray = ["John Doe", "Jane Toe"]
		emailsArray = ["john@gmail.com", "jane@hotmail.com"]
		expectedOutput = "John Doe (john@gmail.com)\n\nJane Toe (jane@hotmail.com)"
		
		output = files.getNamesAndEmailsVerticallySpaced(authorsArray, emailsArray)
		
		self.assertEqual(output, expectedOutput)
	
	def test_getAuthorsNameLine(self):
		emptyNameLine = files.getAuthorsNameLine([])
		singleNameLine = files.getAuthorsNameLine(["John Doe"])
		doubleNameLine = files.getAuthorsNameLine(["John Doe", "Jane Toe"])
		tripleNameLine = files.getAuthorsNameLine(["John Doe", "Jane Toe", "Bob Smith"])
		
		self.assertEqual(emptyNameLine, "")
		self.assertEqual(singleNameLine, "\n    author='John Doe',")
		self.assertEqual(doubleNameLine, "\n    author='John Doe and Jane Toe',")
		self.assertEqual(tripleNameLine, "\n    author='John Doe, Jane Toe, and Bob Smith',")
	
	def test_getAuthorNamesInline(self):
		noNamesLine = files.getAuthorNamesInline([])
		oneNameLine = files.getAuthorNamesInline(["John Doe"])
		twoNamesLine = files.getAuthorNamesInline(["John Doe", "Jane Toe"])
		threeNamesLine = files.getAuthorNamesInline(["John Doe", "Jane Toe", "Bob Smith"])
		
		self.assertEqual(noNamesLine, "")
		self.assertEqual(oneNameLine, "John Doe")
		self.assertEqual(twoNamesLine, "John Doe and Jane Toe")
		self.assertEqual(threeNamesLine, "John Doe, Jane Toe, and Bob Smith")
	
	def test_getAuthorEmailLine(self):
		noEmailOutput = files.getAuthorEmailLine("")
		emailOutput = files.getAuthorEmailLine("john@gmail.com")
		
		self.assertEqual(noEmailOutput, "")
		self.assertEqual(emailOutput, "\n    author_email='john@gmail.com',")
	
	def test_getDescriptionLine(self):
		noDescriptionOutput = files.getDescriptionLine("")
		descriptionOutput = files.getDescriptionLine("fake description")
		
		self.assertEqual(noDescriptionOutput, "")
		self.assertEqual(descriptionOutput, "\n    description='fake description',")
	
	def test_getGithubInfo_returnsListOfEmptyStrings_whenUsernameIsEmptyString(self):
		self.assertEqual(files.getGithubInfo("", "testProjectName"), ["", ""])
	
	def test_getClassifiers_returnsEmptyString_whenClassifierIsDefaultOrNotApplicable(self):
		self.assertEqual(files.getClassifiers("N/A"), "")
		self.assertEqual(files.getClassifiers("Select..."), "")

if __name__ == "__main__":
	unittest.main()



import os
import subprocess
from pypimaker.ui.DialogWindow import ActionDialogWindow

RESET_SELECTION_TEXT = "This will remove generated files.\nAre you sure you want to reset?"

def reset(topLevelDirectory):
    ActionDialogWindow("Are you sure?", RESET_SELECTION_TEXT, negativeAction=exit, mainWindow=True)
    
    filesToRemove = ["LICENSE", "README.md", "requirements.txt", "setup.py", ".gitignore"]
    foldersToRemove = ["build", "dist"]
    for item in os.listdir(topLevelDirectory):
        if item.endswith("egg-info"):
            foldersToRemove.append(item)
    
    for file in filesToRemove:
        commandArray = ["rm", topLevelDirectory + "/" + file]
        subprocess.run(commandArray, capture_output=True)
    
    for folder in foldersToRemove:
        commandArray = ["rm", "-r", topLevelDirectory + "/" + folder]
        subprocess.run(commandArray, capture_output=True)



import os
import tkinter as tk
from pypimaker.ui.DialogWindow import ActionDialogWindow, InfoDialogWindow

MISSING_AUTHOR_NAMES_TEXT = "Some emails have missing author names.\nPlease fix and resubmit."
MISSING_FIRST_NAME_TEXT = "Corresponding author name not given.\nSubmit with no authors?"
MISSING_FIRST_AUTHOR_EMAIL_TEXT = "Email given for other(s), but not first author.\n\
Use next email in list for correspondence?"

class AuthorInfoSelector:
	def __init__(self):
		self.window = tk.Tk()
		self.window.title("PyPI Maker")
		centerX = (self.window.winfo_screenwidth() - 650) // 2
		centerY = (self.window.winfo_screenheight() - 310) // 2
		self.window.geometry(f"650x310+{centerX}+{centerY}")
		
		self.names = []
		self.emails = []
		self.correspondingEmail = ""
		
		labelPrimary = tk.Label(self.window, text="Corresponding Author")
		
		authorFrame1 = tk.Frame(self.window)
		labelName1 = tk.Label(self.window, text="Name:")
		self.entryName1 = tk.Entry(self.window, width=25)
		labelEmail1 = tk.Label(self.window, text="Email:")
		self.entryEmail1 = tk.Entry(self.window, width=25)
		labelName1.pack(in_=authorFrame1, side=tk.LEFT)
		self.entryName1.pack(in_=authorFrame1, side=tk.LEFT)
		labelEmail1.pack(in_=authorFrame1, side=tk.LEFT)
		self.entryEmail1.pack(in_=authorFrame1, side=tk.LEFT)
		
		labelOthers = tk.Label(self.window, text="Other Authors")
		
		authorFrame2 = tk.Frame(self.window)
		labelName2 = tk.Label(self.window, text="Name:")
		self.entryName2 = tk.Entry(self.window, width=25)
		labelEmail2 = tk.Label(self.window, text="Email:")
		self.entryEmail2 = tk.Entry(self.window, width=25)
		labelName2.pack(in_=authorFrame2, side=tk.LEFT)
		self.entryName2.pack(in_=authorFrame2, side=tk.LEFT)
		labelEmail2.pack(in_=authorFrame2, side=tk.LEFT)
		self.entryEmail2.pack(in_=authorFrame2, side=tk.LEFT)
		
		authorFrame3 = tk.Frame(self.window)
		labelName3 = tk.Label(self.window, text="Name:")
		self.entryName3 = tk.Entry(self.window, width=25)
		labelEmail3 = tk.Label(self.window, text="Email:")
		self.entryEmail3 = tk.Entry(self.window, width=25)
		labelName3.pack(in_=authorFrame3, side=tk.LEFT)
		self.entryName3.pack(in_=authorFrame3, side=tk.LEFT)
		labelEmail3.pack(in_=authorFrame3, side=tk.LEFT)
		self.entryEmail3.pack(in_=authorFrame3, side=tk.LEFT)
		
		authorFrame4 = tk.Frame(self.window)
		labelName4 = tk.Label(self.window, text="Name:")
		self.entryName4 = tk.Entry(self.window, width=25)
		labelEmail4 = tk.Label(self.window, text="Email:")
		self.entryEmail4 = tk.Entry(self.window, width=25)
		labelName4.pack(in_=authorFrame4, side=tk.LEFT)
		self.entryName4.pack(in_=authorFrame4, side=tk.LEFT)
		labelEmail4.pack(in_=authorFrame4, side=tk.LEFT)
		self.entryEmail4.pack(in_=authorFrame4, side=tk.LEFT)
		
		authorFrame5 = tk.Frame(self.window)
		labelName5 = tk.Label(self.window, text="Name:")
		self.entryName5 = tk.Entry(self.window, width=25)
		labelEmail5 = tk.Label(self.window, text="Email:")
		self.entryEmail5 = tk.Entry(self.window, width=25)
		labelName5.pack(in_=authorFrame5, side=tk.LEFT)
		self.entryName5.pack(in_=authorFrame5, side=tk.LEFT)
		labelEmail5.pack(in_=authorFrame5, side=tk.LEFT)
		self.entryEmail5.pack(in_=authorFrame5, side=tk.LEFT)
		
		authorFrame6 = tk.Frame(self.window)
		labelName6 = tk.Label(self.window, text="Name:")
		self.entryName6 = tk.Entry(self.window, width=25)
		labelEmail6 = tk.Label(self.window, text="Email:")
		self.entryEmail6 = tk.Entry(self.window, width=25)
		labelName6.pack(in_=authorFrame6, side=tk.LEFT)
		self.entryName6.pack(in_=authorFrame6, side=tk.LEFT)
		labelEmail6.pack(in_=authorFrame6, side=tk.LEFT)
		self.entryEmail6.pack(in_=authorFrame6, side=tk.LEFT)
		
		authorFrame7 = tk.Frame(self.window)
		labelName7 = tk.Label(self.window, text="Name:")
		self.entryName7 = tk.Entry(self.window, width=25)
		labelEmail7 = tk.Label(self.window, text="Email:")
		self.entryEmail7 = tk.Entry(self.window, width=25)
		labelName7.pack(in_=authorFrame7, side=tk.LEFT)
		self.entryName7.pack(in_=authorFrame7, side=tk.LEFT)
		labelEmail7.pack(in_=authorFrame7, side=tk.LEFT)
		self.entryEmail7.pack(in_=authorFrame7, side=tk.LEFT)
		
		authorFrame8 = tk.Frame(self.window)
		labelName8 = tk.Label(self.window, text="Name:")
		self.entryName8 = tk.Entry(self.window, width=25)
		labelEmail8 = tk.Label(self.window, text="Email:")
		self.entryEmail8 = tk.Entry(self.window, width=25)
		labelName8.pack(in_=authorFrame8, side=tk.LEFT)
		self.entryName8.pack(in_=authorFrame8, side=tk.LEFT)
		labelEmail8.pack(in_=authorFrame8, side=tk.LEFT)
		self.entryEmail8.pack(in_=authorFrame8, side=tk.LEFT)
		
		self.buttonDone = tk.Button(self.window, text="Done", command=self.checkDone, fg="blue")
		
		labelPrimary.pack()
		authorFrame1.pack()
		labelOthers.pack()
		authorFrame2.pack()
		authorFrame3.pack()
		authorFrame4.pack()
		authorFrame5.pack()
		authorFrame6.pack()
		authorFrame7.pack()
		authorFrame8.pack()
		self.buttonDone.pack()
		
		self.window.protocol("WM_DELETE_WINDOW", quit)
		self.window.bind("<Return>", self.checkDoneWithReturnKey)
		
		self.window.mainloop()
	
	def checkDone(self):
		names = [self.entryName1.get(), self.entryName2.get(), self.entryName3.get(), 
			self.entryName4.get(), self.entryName5.get(), self.entryName6.get(), 
			self.entryName7.get(), self.entryName8.get()]
		emails = [self.entryEmail1.get(), self.entryEmail2.get(), self.entryEmail3.get(), 
			self.entryEmail4.get(), self.entryEmail5.get(), self.entryEmail6.get(), 
			self.entryEmail7.get(), self.entryEmail8.get()]
		
		noEmailsEntered = True
		for name, email in zip(names, emails):
			if email != "":
				noEmailsEntered = False
				if name == "":
					InfoDialogWindow("Invalid entry", MISSING_AUTHOR_NAMES_TEXT)
					return
		
		if names[0] == "":
			ActionDialogWindow("Invalid entry", MISSING_FIRST_NAME_TEXT, 
				positiveAction=self.doneWithNoAuthors)
			return
		
		if emails[0] == "" and not noEmailsEntered:
			ActionDialogWindow("Invalid entry", MISSING_FIRST_AUTHOR_EMAIL_TEXT)
		
		self.done(names, emails)
	
	def checkDoneWithReturnKey(self, event):        
		self.checkDone()
	
	def done(self, names, emails):
		self.names, self.emails = self.getFilteredNamesAndEmails(names, emails)
		self.correspondingEmail = self.getCorrespondingEmail()
		self.window.destroy()
		self.window.quit()
	
	def doneWithNoAuthors(self):
		self.names = []
		self.emails = []
		self.window.destroy()
		self.window.quit()
	
	def getCorrespondingEmail(self):
		for email in self.emails:
			if email != "":
				return email
	
	def getFilteredNamesAndEmails(self, names, emails):
		filteredNames = []
		filteredEmails = []
		for index, name in enumerate(names):
			if name != "":
				filteredNames.append(names[index])
				filteredEmails.append(emails[index])
		return filteredNames, filteredEmails



from setuptools import setup, find_packages
import pathlib

here = pathlib.Path(__file__).parent.resolve()

long_description = (here / 'README.md').read_text(encoding='utf-8')

setup(
	name='pypimaker', # Required
    version='0.1.10', # Required 
    description='Software designed for simplifying PyPI Python package setups',
	long_description=long_description,
	long_description_content_type='text/markdown',
    url='https://github.com/bdavis222/pypimaker',
    author='Brian Davis',
	classifiers=[ # Defined at https://pypi.org/classifiers/
		'Intended Audience :: Developers',
		'License :: OSI Approved :: MIT License',
		'Programming Language :: Python :: 3',
		'Operating System :: Unix',
		'Operating System :: MacOS :: MacOS X',
		'Operating System :: Microsoft :: Windows'
	],
	packages=find_packages(), # Required
	py_modules=['tests', 'pypimaker.ui', 'pypimaker'], # Generated
	python_requires='>=3.7, <4', 
	project_urls={
		'Bug Reports': 'https://github.com/bdavis222/pypimaker/issues',
		'Funding': 'https://www.paypal.com/donate/?business=UA5NL9MJSFMVY',
		'Source': 'https://github.com/bdavis222/',
	},
	entry_points={
		'console_scripts': [
			'pypimaker = pypimaker.__main__:main'
		]
	}
)


import os
import tkinter as tk
from tkinter import filedialog

SELECT_FOLDER_TEXT = "Select the top-level folder containing your Python project"

def getDirectory():
	filepath = ""
	tk.Tk().withdraw()
	chosenFolder = filedialog.askdirectory(initialdir=filepath, title=SELECT_FOLDER_TEXT)
	if chosenFolder not in ["", " ", "/"] and hasPyFiles(chosenFolder):
		filepath = chosenFolder
		return filepath
	else:
		quit()

def hasPyFiles(filepath):
	for _, _, files in os.walk(filepath):
		for file in files:
			if file.endswith(".py"):
				 return True
	return False
